@@ -1,15 +1,9 @@
-"""Recursively deserialize a Python Sequence, Mapping, or primitive into a.
-
-* dataclass
-* NamedTuple
-* list
-* tuple
-* other supported structure
-"""
+"""Load loosely-typed objects into strongly-typed containers."""
 
 import inspect
 import itertools
 from dataclasses import InitVar, dataclass, field, fields, is_dataclass
+from enum import Enum
 from typing import _TypedDictMeta  # type: ignore
 from typing import (
     Any,
@@ -19,8 +13,6 @@ from typing import (
     List,
     Literal,
     Mapping,
-    NoReturn,
-    Optional,
     Sequence,
     Tuple,
     Type,
@@ -32,6 +24,7 @@ from typing import (
 )
 
 from .errors import DepthContainer, DeserializeError
+from .overrides import DEFAULT_OVERRIDE, Override, get_override
 from .typedefs import (
     MISSING,
     NO_RESULT,
@@ -52,7 +45,6 @@ def load(
     obj: Any,
     constructor: Type[T],
     typesafe_constructor: bool = True,
-    convert_primitives: bool = True,
 ) -> T:
     """Deserialize an object into its constructor.
 
@@ -60,12 +52,6 @@ def load(
     :param constructor: the type into which we want serialize 'obj'
     :param typesafe_constructor: makes sure that the provided top-level
         constructor is not one of several "unsafe" types
-    :param convert_primitives: automatically convert primitive values, if
-        encountered. Even if enabled here, this becomes automatically disabled,
-        recursively downward, if an ambiguous container type is encountered. At
-        this time, `Union` is considered ambiguous. Note: according to the type
-        system, `Optional` is considered a `Union` type, but this special case
-        does not disable automatic conversion (special-cased code handles).
 
     :returns: an instance of your constructor, recursively filled
 
@@ -81,36 +67,27 @@ def load(
         obj=obj,
         constructor=constructor,
         depth=[],
-        convert_primitives=convert_primitives,
     ).run()
 
 
 @dataclass
 class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
-    """Deserialize the type.
+    """Deserialize an object into a more-strongly-typed form.
 
     :attr depth: keeps track of recursive position. Supremely helpful for
         error messages, since it shows the user exactly where the parsing
         fails.
-    :attr dataclass_validate: argument provided only by upstream dataclasses.
-        An internal implementation detail, but interesting nonetheless.
-        Function either returns `True` on positive validation / `False` on
-        non-validation, or returns nothing at all and instead relies on the
-        raising of exceptions to indicate whether validation passed for failed.
 
-    NOTE: we use typing.get_type_hints because it correctly handles
-    ForwardRefs, translating string references into their correct type in
-    strange and mysterious ways.
+    NOTE: throughout this class, we use typing.get_type_hints because it
+    correctly handles ForwardRefs, translating string references into their
+    correct type in strange and mysterious ways.
     """
 
     obj: Any
     constructor: Type[T]
     depth: InitVar[List[DepthContainer]]
-    convert_primitives: bool
     key: Any = field(default=MISSING)
-    dataclass_validate: Optional[
-        Union[Callable[[Any], NoReturn], Callable[[Any], bool]]
-    ] = field(default=None)
+    dataclass_override: Override = field(default=DEFAULT_OVERRIDE)
     new_depth: List[DepthContainer] = field(init=False)
     constructor_args: Tuple[Type, ...] = field(init=False)
     constructor_origin: Type = field(init=False)
@@ -129,6 +106,7 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
         self.check_functions = (
             self._check_any,
             self._check_literal,
+            self._check_enum,
             self._check_primitive,
             self._check_none,
             self._check_undefined,
@@ -141,6 +119,7 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
             self._check_initvar_instance,
             self._check_union,
             self._check_typevar,
+            self._check_isinstance,
         )
 
     def run(self) -> T:
@@ -148,8 +127,9 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
 
         Validate dataclass fields where specified by the user.
         """
+        self._transform_load()
         try:
-            result = next(
+            result: T = next(  # type: ignore
                 itertools.dropwhile(
                     _is_no_result,
                     (function() for function in self.check_functions),
@@ -174,30 +154,66 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                     message_override=str(error),
                 ) from error
             raise
-        if not self.dataclass_validate is None:
-            try:
-                validation_result = self.dataclass_validate(result)
-            except Exception as error:
-                if not isinstance(error, DeserializeError):
-                    raise DeserializeError(
-                        self.constructor,
-                        self.obj,
-                        self.new_depth,
-                        self.key,
-                        message_override=str(error),
-                    ) from error
-                raise
-            if validation_result is False:
+        self._validate(result)
+        return self._transform_postload(result)
+
+    def _validate(self, result: Any) -> None:
+        """Verify that a result is correct."""
+        try:
+            validation_result = self.dataclass_override.validate(result)
+        except Exception as error:
+            if not isinstance(error, DeserializeError):
                 raise DeserializeError(
                     self.constructor,
                     self.obj,
                     self.new_depth,
                     self.key,
-                    message_override=(
-                        f"{repr(self.dataclass_validate)} returned False"
-                    ),
-                )
-        return result  # type: ignore
+                    message_override=str(error),
+                ) from error
+            raise
+        if validation_result is False:
+            raise DeserializeError(
+                self.constructor,
+                self.obj,
+                self.new_depth,
+                self.key,
+                message_override=(
+                    f"{repr(self.dataclass_override.validate)} returned False"
+                ),
+            )
+
+    def _transform_load(self) -> None:
+        """Modify `self.obj` inplace if dataclass_transform is defined."""
+        try:
+            self.obj = self.dataclass_override.transform_load(self.obj)
+        except Exception as error:
+            if not isinstance(error, DeserializeError):
+                raise DeserializeError(
+                    self.constructor,
+                    self.obj,
+                    self.new_depth,
+                    self.key,
+                    message_override=str(error),
+                ) from error
+            raise
+
+    def _transform_postload(self, result: T) -> T:
+        """Modify result if transform_postload is defined.
+
+        TODO: figure out why type below needs to be ignored
+        """
+        try:
+            return self.dataclass_override.transform_postload(result)
+        except Exception as error:
+            if not isinstance(error, DeserializeError):
+                raise DeserializeError(
+                    self.constructor,
+                    self.obj,
+                    self.new_depth,
+                    self.key,
+                    message_override=str(error),
+                ) from error
+            raise
 
     def _check_dataclass(self) -> PossibleResult[T]:
         """Checks whether a result is a dataclass."""
@@ -207,16 +223,18 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                     Mapping, self.obj, self.new_depth, self.key
                 )
             parameters = inspect.signature(self.constructor).parameters
-            field_meta = {f.name: f.metadata for f in fields(self.constructor)}
+            field_meta = {
+                f.name: get_override(f.metadata.get("serdelicacy"))
+                for f in fields(self.constructor)
+            }
             return self.constructor(
                 **{
                     name: Deserialize(
                         obj=self.obj.get(name, MISSING),
                         constructor=_type,
                         depth=self.new_depth,
-                        convert_primitives=self.convert_primitives,
                         key=name,
-                        dataclass_validate=field_meta[name].get("validate"),
+                        dataclass_override=field_meta[name],
                     ).run()
                     for name, _type in get_type_hints(self.constructor).items()
                     if not (
@@ -242,7 +260,6 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                         obj=self.obj.get(name, MISSING),
                         constructor=_type,
                         depth=self.new_depth,
-                        convert_primitives=self.convert_primitives,
                         key=name,
                     ).run()
                     for name, _type in get_type_hints(self.constructor).items()
@@ -275,7 +292,6 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                         obj=value,
                         constructor=self.constructor_args[0],
                         depth=self.new_depth,
-                        convert_primitives=self.convert_primitives,
                     ).run()
                     for value in self.obj
                 )  # type: ignore
@@ -292,7 +308,6 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                     obj=self.obj[i],
                     constructor=arg,
                     depth=self.new_depth,
-                    convert_primitives=self.convert_primitives,
                 ).run()
                 for i, arg in enumerate(self.constructor_args)
             )  # type: ignore
@@ -329,7 +344,6 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                     obj=value,
                     constructor=_arg,
                     depth=self.new_depth,
-                    convert_primitives=self.convert_primitives,
                 ).run()
                 for value in self.obj
             )  # type: ignore
@@ -360,14 +374,12 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                         obj=key,
                         constructor=_tpkey,
                         depth=self.new_depth,
-                        convert_primitives=self.convert_primitives,
                         key=key,
                     )
                     .run(): Deserialize(
                         obj=value,
                         constructor=_tpvalue,
                         depth=self.new_depth,
-                        convert_primitives=self.convert_primitives,
                         key=key,
                     )
                     .run()
@@ -390,7 +402,6 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                     obj=self.obj.get(name, MISSING),
                     constructor=_type,
                     depth=self.new_depth,
-                    convert_primitives=self.convert_primitives,
                     key=name,
                 ).run()
                 for name, _type in get_type_hints(self.constructor).items()
@@ -404,7 +415,6 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                 obj=self.obj,
                 constructor=self.constructor.type,  # type: ignore
                 depth=self.new_depth,
-                convert_primitives=self.convert_primitives,
             ).run()
         return NO_RESULT
 
@@ -433,33 +443,12 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
         return NO_RESULT
 
     def _check_primitive(self) -> PossibleResult[T]:
-        """Check if result is a primitive.
-
-        If `self.convert_primitives` is `True`, tries to automatically
-        convert primitive value to the specified type.
-
-        `None` and `MISSING` are both considered "unconvertable"
-        """
+        """Check if result is a primitive."""
         if self.constructor in _PRIMITIVES:
-            if self.obj is MISSING:
-                raise DeserializeError(
-                    self.constructor, self.obj, self.new_depth, self.key
-                )
-            if self.obj is None:
-                raise DeserializeError(
-                    self.constructor, self.obj, self.new_depth, self.key
-                )
             if not isinstance(self.obj, self.constructor):
-                if not self.convert_primitives:
-                    raise DeserializeError(
-                        self.constructor, self.obj, self.new_depth, self.key
-                    )
-                try:
-                    return self.constructor(self.obj)  # type: ignore
-                except (ValueError, TypeError) as error:
-                    raise DeserializeError(
-                        self.constructor, self.obj, self.new_depth, self.key
-                    ) from error
+                raise DeserializeError(
+                    self.constructor, self.obj, self.new_depth, self.key
+                )
             return self.obj
         return NO_RESULT
 
@@ -484,6 +473,17 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
             )
         return NO_RESULT
 
+    def _check_enum(self) -> PossibleResult[T]:
+        """Check if a result is an Enum type.
+
+        Must match the enum name exactly.
+        """
+        if isinstance(self.constructor, type) and issubclass(
+            self.constructor, Enum
+        ):
+            return self.constructor(self.obj)  # type: ignore
+        return NO_RESULT
+
     def _check_any(self) -> PossibleResult[T]:
         """Check if result is typing.Any."""
         if _is_any(self.constructor):
@@ -491,13 +491,7 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
         return NO_RESULT
 
     def _check_union(self) -> PossibleResult[T]:
-        """Check if result is a Union.
-
-        Encountering this type usually recursively disables
-        `convert_primitives` for items found within the `Union`. The
-        exception are `Optional` types, which are really `Union[Any,
-        None]` and common enough for us to find a workaround.
-        """
+        """Check if result is a Union."""
         if _is_union(self.constructor):
             args = get_args(self.constructor)
             is_optional = len(args) == 2 and type(None) in args
@@ -507,16 +501,11 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
             if is_optional_property and self.obj is MISSING:
                 return MISSING  # type: ignore
             for argument in args:
-                convert_primitives = self.convert_primitives and (
-                    (is_optional and argument != type(None))
-                    or (is_optional_property and argument != Missing)
-                )
                 try:
                     return Deserialize(
                         obj=self.obj,
                         constructor=argument,
                         depth=self.new_depth,
-                        convert_primitives=convert_primitives,
                     ).run()
                 except DeserializeError:
                     pass
@@ -536,10 +525,21 @@ class Deserialize(Generic[T]):  # pylint: disable=too-many-instance-attributes
                     else object
                 ),
                 depth=self.new_depth,
-                convert_primitives=self.convert_primitives,
             ).run()
         return NO_RESULT
 
+    def _check_isinstance(self) -> PossibleResult[T]:
+        """Final check to see if the result is an instance of its type.
+
+        Some types aren't checkable with `isinstance`, but this should
+        catch most extra cases outside the typing library.
+        """
+        try:
+            is_exact_match = isinstance(self.obj, self.constructor)
+        except Exception:  # pylint: disable=broad-except
+            return NO_RESULT
+        return self.obj if is_exact_match else NO_RESULT
+
 
 def _is_initvar_instance(typeval: Type) -> bool:
     """Check if a type is an InitVar with a type inside."""
@@ -571,5 +571,3 @@ _TYPE_UNSAFE_CHECKS = (
 _PRIMITIVES = {str, int, float, bool}
 
 _ANY = {Any, object, InitVar}
-
-_UNION_PRIMITIVE_ALLOW_CONVERSION = {type(None), Missing}
